<?php

namespace WPSpeedo_Team;

if ( ! defined( 'ABSPATH' ) ) exit;

final class Admin {

    public function __construct() {
        
        add_action( 'admin_menu', [ $this, 'register_admin_menu' ] );

        $this->maybe_create_db_table();

        $this->load_editor_template();

        return $this;

    }

    public function load_editor_template() {

        if ( ! Helpers::is_shortcode_preview() ) return;

        add_action( 'template_redirect', function() {
            
            new Shortcode_Loader([
                'id' => 'preview',
                'settings' => (array) Helpers::get_temp_settings(),
                'mode' => 'preview'
            ]);

            die();
            
        });
		
        
        add_action( 'show_admin_bar', '__return_false' );

    }

    public function add_shortcode_body_class( $classes ) {

        if ( Helpers::is_shortcode_preview() ) return array_merge( $classes, array( 'gs-sm-sec-shortcode-preview--page' ) );

        return $classes;

    }

    public function register_admin_menu() {
        
        $capability             = 'manage_options';
        $shortcode_menu_title   = __('Shortcodes', 'wpspeedo-team');
        $settings_menu_title    = __('Settings', 'wpspeedo-team');
        $order_menu_title       = __('Custom Order', 'wpspeedo-team');

        add_submenu_page( Helpers::get_top_label_menu(), $shortcode_menu_title, $shortcode_menu_title, $capability, 'wps-team', [ $this, 'plugin_admin_page' ], 50 );
        add_submenu_page( Helpers::get_top_label_menu(), $settings_menu_title, $settings_menu_title, $capability, 'wps-team#/settings', [ $this, 'plugin_admin_page' ], 60 );
        add_submenu_page( Helpers::get_top_label_menu(), $order_menu_title, $order_menu_title, $capability, 'wps-team#/custom-order', [ $this, 'plugin_admin_page' ], 70 );

    }

    public function plugin_admin_page() {

        ?>

        <div class="wpspeedo--plugin-wrap wpspeedo--team-members-wrap">
            <div class="wpspeedo--app-container">
                <div id="wpspeedo--app">
                    <router-view :key="$route.fullPath"></router-view>
                </div>
            </div>
        </div>

        <?php

    }

    public function maybe_create_db_table() {

        global $wpdb;

        $wps_team_db_ver = '1.0';

        if ( get_option("{$wpdb->prefix}wps_team_db_ver") == $wps_team_db_ver ) return;
        
        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

        $sql = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}wps_team (
            id BIGINT(20) unsigned NOT NULL AUTO_INCREMENT,
            name TEXT NOT NULL,
            settings LONGTEXT NOT NULL,
            created_at DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00',
            updated_at DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00',
            PRIMARY KEY (id)
        )".$wpdb->get_charset_collate().";";
            
        if ( get_option("{$wpdb->prefix}wps_team_db_ver") < $wps_team_db_ver ) dbDelta( $sql );

        update_option( "{$wpdb->prefix}wps_team_db_ver", $wps_team_db_ver );
        
    }

}