<?php

namespace WPSpeedo_Team;

if ( !defined( 'ABSPATH' ) ) {
    exit;
}
class Assets
{
    public function __construct()
    {
        add_action( 'wp_enqueue_scripts', [ $this, 'frontend_scripts' ] );
        add_action( 'admin_enqueue_scripts', [ $this, 'admin_scripts' ] );
        add_action( 'admin_enqueue_scripts', [ $this, 'meta_box_scripts' ], 999 );
        add_action( 'wp_head', [ $this, 'load_google_fonts' ] );
    }
    
    public function fontawesome_scripts( $context = 'public' )
    {
        wp_register_style(
            'wpspeedo-fontawesome',
            WPS_TEAM_ASSET_URL . 'libs/fontawesome/css/fontawesome.min.css',
            '',
            WPS_TEAM_VERSION
        );
        wp_register_style(
            'wpspeedo-fontawesome--all',
            WPS_TEAM_ASSET_URL . 'libs/fontawesome/css/all.min.css',
            '',
            WPS_TEAM_VERSION
        );
        wp_register_style(
            'wpspeedo-fontawesome--brands',
            WPS_TEAM_ASSET_URL . 'libs/fontawesome/css/brands.min.css',
            '',
            WPS_TEAM_VERSION
        );
        wp_register_style(
            'wpspeedo-fontawesome--regular',
            WPS_TEAM_ASSET_URL . 'libs/fontawesome/css/regular.min.css',
            '',
            WPS_TEAM_VERSION
        );
        wp_register_style(
            'wpspeedo-fontawesome--solid',
            WPS_TEAM_ASSET_URL . 'libs/fontawesome/css/solid.min.css',
            '',
            WPS_TEAM_VERSION
        );
        wp_register_style(
            'wpspeedo-fontawesome--svg-with-js',
            WPS_TEAM_ASSET_URL . 'libs/fontawesome/css/svg-with-js.min.css',
            '',
            WPS_TEAM_VERSION
        );
        wp_register_style(
            'wpspeedo-fontawesome--shims',
            WPS_TEAM_ASSET_URL . 'libs/fontawesome/css/v4-shims.min.css',
            '',
            WPS_TEAM_VERSION
        );
    }
    
    public function get_fontawesome_dependencies( $context = 'public' )
    {
        $dependencies = [];
        
        if ( $context == 'public' ) {
            $dependencies[] = 'wpspeedo-fontawesome--all';
        } else {
            $dependencies[] = 'wpspeedo-fontawesome--all';
        }
        
        return $dependencies;
    }
    
    public function admin_scripts( $hook )
    {
        if ( Helpers::post_type_name() . '_page_wps-team' != $hook ) {
            return;
        }
        $this->fontawesome_scripts( 'plugin' );
        $dependencies = [];
        $dependencies = array_merge( $dependencies, $this->get_fontawesome_dependencies( 'plugin' ) );
        wp_enqueue_style(
            'wpspeedo-team-admin',
            WPS_TEAM_ADMIN_ASSET_URL . 'css/style.min.css',
            $dependencies,
            WPS_TEAM_VERSION
        );
        $shortcode_editor = new Shortcode_Editor();
        $settings_editor = new Settings_Editor();
        $data = array(
            'nonce'     => wp_create_nonce( '_wpspeedo_team_nonce' ),
            'ajaxurl'   => admin_url( 'admin-ajax.php' ),
            'adminurl'  => admin_url(),
            'siteurl'   => home_url(),
            'pluginurl' => WPS_TEAM_URL,
            'action'    => 'wpspeedo_team_ajax_handler',
            'fields'    => $shortcode_editor->get_controls(),
            'tabs'      => plugin()->tabs,
            'settings'  => $settings_editor->get_controls(),
            'icon_data' => Icon_Manager::get_icon_manager_tabs_config(),
        );
        $is_whitelabeled = wps_team_fs()->is_whitelabeled();
        $has_paid_plan = wps_team_fs()->apply_filters( 'has_paid_plan_account', wps_team_fs()->has_paid_plan() );
        $is_paying = wps_team_fs()->is_paying();
        $is_paid_trial = wps_team_fs()->is_paid_trial();
        $data['show_upgrade'] = !$is_whitelabeled && $has_paid_plan && !$is_paying && !$is_paid_trial;
        $data['translations'] = $this->get_translation_strings();
        $data['demo_data_status'] = Helpers::get_demo_data_status();
        wp_register_script(
            'wpspeedo-team-admin',
            WPS_TEAM_ADMIN_ASSET_URL . 'js/script.min.js',
            [ 'jquery', 'underscore', 'jquery-ui-sortable' ],
            WPS_TEAM_VERSION,
            true
        );
        wp_localize_script( 'wpspeedo-team-admin', '_wps_team_data', $data );
        wp_enqueue_script( 'wpspeedo-team-admin' );
    }
    
    public function meta_box_scripts( $hook )
    {
        if ( !in_array( $hook, [ 'post.php', 'post-new.php' ] ) ) {
            return;
        }
        $this->fontawesome_scripts( 'meta_box' );
        $dependencies = [];
        $dependencies = array_merge( $dependencies, $this->get_fontawesome_dependencies( 'meta_box' ) );
        wp_enqueue_style(
            'wpspeedo-team--meta-box',
            WPS_TEAM_ADMIN_ASSET_URL . 'css/meta-box.min.css',
            $dependencies,
            WPS_TEAM_VERSION
        );
        $meta_box_editor = new Meta_Box_Editor();
        $data = [
            'nonce'     => wp_create_nonce( '_wpspeedo_team_nonce' ),
            'ajaxurl'   => admin_url( 'admin-ajax.php' ),
            'adminurl'  => admin_url(),
            'siteurl'   => home_url(),
            'action'    => 'wpspeedo_team_ajax_handler',
            'fields'    => $meta_box_editor->get_controls(),
            'icon_data' => Icon_Manager::get_icon_manager_tabs_config(),
        ];
        // wp_enqueue_media();
        wp_register_script(
            'wpspeedo-team--meta-box',
            WPS_TEAM_ADMIN_ASSET_URL . 'js/meta-box.min.js',
            [ 'jquery', 'underscore' ],
            WPS_TEAM_VERSION,
            true
        );
        wp_localize_script( 'wpspeedo-team--meta-box', '_wps_team_data', $data );
        wp_enqueue_script( 'wpspeedo-team--meta-box' );
    }
    
    public function frontend_scripts()
    {
        $dep_css = [];
        $dep_js = [ 'jquery' ];
        $this->fontawesome_scripts( 'public' );
        $dep_css = array_merge( $dep_css, $this->get_fontawesome_dependencies( 'public' ) );
        $dep_css[] = 'wpspeedo-swiper';
        $dep_js[] = 'wpspeedo-swiper';
        $dep_js[] = 'wpspeedo-isotope';
        wp_register_style(
            'wpspeedo-swiper',
            WPS_TEAM_ASSET_URL . 'libs/swiper/swiper-bundle.min.css',
            [],
            WPS_TEAM_VERSION
        );
        wp_register_script(
            'wpspeedo-swiper',
            WPS_TEAM_ASSET_URL . 'libs/swiper/swiper-bundle.min.js',
            [],
            WPS_TEAM_VERSION,
            true
        );
        wp_register_script(
            'wpspeedo-isotope',
            WPS_TEAM_ASSET_URL . 'libs/isotope/isotope.min.js',
            [],
            WPS_TEAM_VERSION,
            true
        );
        wp_register_style(
            'wpspeedo-team',
            WPS_TEAM_ASSET_URL . 'css/style.min.css',
            $dep_css,
            WPS_TEAM_VERSION
        );
        wp_register_script(
            'wpspeedo-team',
            WPS_TEAM_ASSET_URL . 'js/script.min.js',
            $dep_js,
            WPS_TEAM_VERSION,
            true
        );
        wp_register_style(
            'wpspeedo-team-preview',
            WPS_TEAM_ASSET_URL . 'admin/css/preview.min.css',
            [],
            WPS_TEAM_VERSION
        );
        wp_register_script(
            'wpspeedo-team-preview',
            WPS_TEAM_ASSET_URL . 'admin/js/preview.min.js',
            [ 'wpspeedo-team', 'underscore' ],
            WPS_TEAM_VERSION,
            true
        );
        wp_enqueue_style( 'wpspeedo-team' );
        wp_enqueue_script( 'wpspeedo-team' );
        
        if ( Helpers::is_shortcode_preview() ) {
            wp_enqueue_style( 'wpspeedo-team-preview' );
            wp_enqueue_script( 'wpspeedo-team-preview' );
        }
    
    }
    
    public function load_google_fonts()
    {
        if ( apply_filters( 'wpspeedo_team/disable_google_fonts', false ) ) {
            return;
        }
        ?>

        <link rel="preconnect" href="https://fonts.googleapis.com">
        <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
        <link href="https://fonts.googleapis.com/css2?family=Cambo&family=Fira+Sans:wght@400;500&family=Roboto:wght@400;500&display=swap" rel="stylesheet">

        <?php 
    }
    
    public function get_translation_strings()
    {
        return require_once WPS_TEAM_INC_PATH . '/translations.php';
    }

}