<?php

namespace WPSpeedo_Team;

if ( ! defined( 'ABSPATH' ) ) exit;

class Settings_Editor extends Editor_Controls {

	public function __construct( array $data = [], array $args = null ) {
		parent::__construct( $data, $args );
		do_action( 'wpspeedo_team/settings_editor/init', $this );
	}

	public function get_name() {
		return 'meta_box_editor';
	}

	protected function _register_controls() {

		$this->general_settings();
		$this->menu_settings();

	}

	protected function general_settings() {

		$this->start_controls_section( 'general_settings_section', [
			'label' => __( 'General Settings', 'wpspeedo-team' ),
		]);

		$this->add_control( 'enable_archive', [
			'label' => __( 'Enable Single/Archive Page', 'wpspeedo-team' ),
			'label_block' => false,
			'separator' => 'none',
			'type' => Controls_Manager::SWITCHER,
			'default' => true
		]);

		$this->add_control( 'post_type_slug', [
			'label' => __( 'Archive Slug', 'wpspeedo-team' ),
			'label_block' => false,
			'separator' => 'none',
			'type' => Controls_Manager::TEXT,
			'default' => Helpers::get_archive_slug(),
			'condition' => [
				'enable_archive' => true
			]
		]);

		$this->add_control( 'enable_group_archive', [
			'label' => __( 'Enable Group Archive', 'wpspeedo-team' ),
			'label_block' => false,
			'separator' => 'none',
			'type' => Controls_Manager::SWITCHER,
			'default' => false,
			'condition' => [
				'enable_archive' => true
			]
		]);

		$this->add_control( 'group_slug', [
			'label' => __( 'Group Slug', 'wpspeedo-team' ),
			'label_block' => false,
			'separator' => 'none',
			'type' => Controls_Manager::TEXT,
			'default' => Helpers::get_group_archive_slug(),
			'condition' => [
				'enable_group_archive' => true,
				'enable_archive' => true
			]
		]);

		$this->end_controls_section();

	}

	protected function menu_settings() {

		$this->start_controls_section( 'menu_settings_section', [
			'label' => __( 'Menu Settings', 'wpspeedo-team' ),
		]);

		$this->add_control( 'member_single_name', [
			'label' => __( 'Member Single Name', 'wpspeedo-team' ),
			'label_block' => false,
			'separator' => 'none',
			'type' => Controls_Manager::TEXT,
			'default' => 'Member'
		]);

		$this->add_control( 'member_plural_name', [
			'label' => __( 'Member Plural Name', 'wpspeedo-team' ),
			'label_block' => false,
			'separator' => 'none',
			'type' => Controls_Manager::TEXT,
			'default' => 'Members'
		]);

		$this->add_control( 'group_single_name', [
			'label' => __( 'Group Single Name', 'wpspeedo-team' ),
			'label_block' => false,
			'separator' => 'none',
			'type' => Controls_Manager::TEXT,
			'default' => 'Group'
		]);

		$this->add_control( 'group_plural_name', [
			'label' => __( 'Group Plural Name', 'wpspeedo-team' ),
			'label_block' => false,
			'separator' => 'none',
			'type' => Controls_Manager::TEXT,
			'default' => 'Groups'
		]);

		$this->end_controls_section();

	}

}