<?php

namespace WPSpeedo_Team;

if ( ! defined( 'ABSPATH' ) ) exit;

class Shortcode_Editor extends Editor_Controls {

	public function __construct( array $data = [], array $args = null ) {
		parent::__construct( $data, $args );
		do_action( 'wpspeedo_team/shortcode_editor/init', $this );
	}

	public function get_name() {
		return 'shortcode_editor';
	}
    
    protected function _register_controls() {

		$this->layout_section();

		$this->carousel_section();

		$this->query_section();

		$this->style_section();

		$this->typo_section();

		$this->advance_section();
        
	}

	protected function layout_section() {

		$this->start_controls_section( 'layout_section', [
			'label' => __( 'Layout', 'wpspeedo-team' ),
		]);

		$this->add_control( 'display_type', [
			'label' => __( 'Display Type', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::SELECT,
			'placeholder' => __( 'Display Type', 'wpspeedo-team' ),
			'render_type' => 'template',
			'options' => Helpers::get_control_options('display_type'),
			'default' => 'grid',
		]);

		$this->add_control( 'theme', [
			'label' => __( 'Theme', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::SELECT,
			'placeholder' => __( 'Theme', 'wpspeedo-team' ),
			'render_type' => 'template',
			'options' => Helpers::get_control_options('theme'),
			'default' => 'theme-one',
		]);

		$this->add_control( 'card_action', [
			'label' => __( 'Card Action', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::SELECT,
			'placeholder' => __( 'Card Action', 'wpspeedo-team' ),
			'render_type' => 'template',
			'options' => Helpers::get_control_options('card_action'),
			'default' => 'single-page',
		]);

		$this->add_responsive_control( 'container_width', [
			'label' => __( 'Container Width', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::SLIDER,

			'size_units' => [ '%', 'px', 'vw' ],
			
			'range' => [
				'%' => [
					'min' => 1,
					'max' => 100,
					'default' => 100,
				],
				'px' => [
					'min' => 1,
					'max' => 2000,
					'default' => 1200,
				],
				'vw' => [
					'min' => 1,
					'max' => 100,
					'default' => 80,
				],
			],
			'unit' => 'px',
			'tablet_unit' => '%',
			'small_tablet_unit' => '%',
			'mobile_unit' => '%',
			'default' => 1200,
			'tablet_default' => 90,
			'small_tablet_default' => 90,
			'mobile_default' => 85,
		]);

		$this->add_responsive_control( 'columns', [
			'label' => __( 'Columns', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::SLIDER,
			'min' => 1,
			'max' => 10,
			'default' => 3,
			'tablet_default' => 3,
			'small_tablet_default' => 2,
			'mobile_default' => 1,
		]);

		$this->add_responsive_control( 'gap', [
			'label' => __( 'Gap', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::SLIDER,
			'min' => 0,
			'max' => 100,
			'step' => 1,
			'default' => 30,
			'tablet_default' => 30,
			'small_tablet_default' => 30,
			'mobile_default' => 30,
		]);

		$this->add_responsive_control( 'gap_vertical', [
			'label' => __( 'Gap Vertical', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::SLIDER,
			'min' => 0,
			'max' => 100,
			'step' => 1,
			'default' => 30,
			'tablet_default' => 30,
			'small_tablet_default' => 30,
			'mobile_default' => 30,
			'condition' => [
				'display_type' => ['grid', 'carousel']
			]
		]);

		$this->end_controls_section();

	}

	protected function carousel_section() {

		$this->start_controls_section( 'carousel_section', [
			'label' => __( 'Carousel', 'wpspeedo-team' ),
			'condition' => [
				'display_type' => ['carousel', 'carousel-ajax']
			]
		]);

		$this->add_control( 'speed', [
			'label' => __( 'Carousel Speed', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::SLIDER,
			'min' => 100,
			'max' => 5000,
			'step' => 100,
			'default' => 800
		]);

		$this->add_control( 'dots', [
			'label' => __( 'Dots Pagination', 'wpspeedo-team' ),
			'label_block' => false,
			'type' => Controls_Manager::SWITCHER,
			'default' => true,
			'render_type' => 'template',
		]);

		$this->add_control( 'navs', [
			'label' => __( 'Arrow Navigation', 'wpspeedo-team' ),
			'label_block' => false,
			'type' => Controls_Manager::SWITCHER,
			'default' => true,
			'render_type' => 'template',
		]);

		$this->add_control( 'loop', [
			'label' => __( 'Carousel Loop', 'wpspeedo-team' ),
			'label_block' => false,
			'type' => Controls_Manager::SWITCHER,
			'default' => true,
			'render_type' => 'template',
		]);

		$this->add_control( 'dynamic_dots', [
			'label' => __( 'Dynamic Dots', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'wheel', [
			'label' => __( 'Scroll Navigation', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'keyboard', [
			'label' => __( 'Keyboard Navigation', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'autoplay', [
			'label' => __( 'Autoplay', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'delay', [
			'label' => __( 'Autoplay Delay', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'pause_on_hover', [
			'label' => __( 'Pause On Hover', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->end_controls_section();

	}

	protected function query_section() {

		$this->start_controls_section( 'query_section', [
			'label' => __( 'Query', 'wpspeedo-team' ),
			'tab' => 'query'
		]);

		$this->add_control( 'show_all', [
			'label' => __( 'Display All Members', 'wpspeedo-team' ),
			'label_block' => false,
			'render_type' => 'template',
			'type' => Controls_Manager::SWITCHER,
			'default' => true,
		]);
		
		$this->add_control( 'limit', [
			'label' => __( 'Members to Display', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::NUMBER,
			'default' => 12,
			'min' => 1,
			'max' => 999,
			'render_type' => 'template',
			'condition' => [
				'show_all' => false
			]
		]);

		$this->add_control( 'orderby', [
			'label' => __( 'Order By', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::SELECT,
			'render_type' => 'template',
			'options' => Helpers::get_control_options('orderby'),
			'default' => 'date',
		]);

		$this->add_control( 'order', [
			'label' => __( 'Order', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::SELECT,
			'render_type' => 'template',
			'options' => [
                [ 'label' => 'Ascending', 'value' => 'ASC' ],
                [ 'label' => 'Descending', 'value' => 'DESC' ],
			],
			'default' => 'DESC',
		]);

		$this->add_control( 'include', [
			'label' => __( 'Include', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'exclude', [
			'label' => __( 'Exclude', 'wpspeedo-team' ),
			'label_block' => true,
			'separator' => 'none',
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->end_controls_section();

	}

	protected function style_section() {

		$this->start_controls_section( 'style_section', [
			'label' => __( 'Style', 'wpspeedo-team' ),
			'tab' => 'style'
		]);

		$this->add_group_control( Group_Control_Background::get_type(), [
			'name' => 'item_background',
			'label' => __( 'Team Background', 'wpspeedo-team' ),
			'types' => [ 'classic', 'gradient' ]
		]);

		$this->add_control( 'title_color', [
			'label' => __( 'Title Color', 'wpspeedo-team' ),
			'label_block' => false,
			'type' => Controls_Manager::COLOR,
			'separator' => 'after'
		]);

		$this->add_control( 'title_color_hover', [
			'label' => __( 'Title Color Hover', 'wpspeedo-team' ),
			'label_block' => false,
			'type' => Controls_Manager::COLOR
		]);

		$this->add_control( 'designation_color', [
			'label' => __( 'Designation Color', 'wpspeedo-team' ),
			'label_block' => false,
			'type' => Controls_Manager::COLOR
		]);

		$this->add_control( 'desc_color', [
			'label' => __( 'Description Color', 'wpspeedo-team' ),
			'label_block' => false,
			'type' => Controls_Manager::COLOR
		]);

		$this->add_control( 'divider_color', [
			'label' => __( 'Divider Color', 'wpspeedo-team' ),
			'label_block' => false,
			'type' => Controls_Manager::COLOR
		]);
		
		$this->add_control( 'social_colors', [
			'label' => __('Social color settings', 'wpspeedo-team'),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE
		]);
		
		$this->add_control( 'skill_colors', [
			'label' => __('Skill color settings', 'wpspeedo-team'),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE
		]);

		$this->end_controls_section();

	}

	protected function typo_section() {

		$this->start_controls_section( 'typo_section', [
			'label' => __( 'Typography', 'wpspeedo-team' ),
			'tab' => 'typo'
		]);

		$this->add_control( 'typo_name', [
			'label' => __( 'Typo: Name', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'typo_content', [
			'label' => __( 'Typo: Content', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'typo_desig', [
			'label' => __( 'Typo: Designation', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'typo_meta', [
			'label' => __( 'Typo: Meta', 'wpspeedo-team' ),
			'label_block' => true,
			'separator' => 'none',
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->end_controls_section();

	}

	protected function advance_section() {

		$this->start_controls_section( 'advance_section', [
			'label' => __( 'Advance', 'wpspeedo-team' ),
			'tab' => 'advance'
		]);

		$this->add_control( 'bg_color', [
			'label' => __( 'Background Color', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'custom_class', [
			'label' => __( 'Custom Class', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'margin', [
			'label' => __( 'Margin', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'padding', [
			'label' => __( 'Padding', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'z_index', [
			'label' => __( 'Z Index', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'box_shadow', [
			'label' => __( 'Box Shadow', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'border', [
			'label' => __( 'Border', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'border_radius', [
			'label' => __( 'Border Radius', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'entrance_animation', [
			'label' => __( 'Entrance Animation', 'wpspeedo-team' ),
			'label_block' => true,
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->add_control( 'hover_animation', [
			'label' => __( 'Hover Animation', 'wpspeedo-team' ),
			'label_block' => true,
			'separator' => 'none',
			'type' => Controls_Manager::UPGRADE_NOTICE,
		]);

		$this->end_controls_section();

	}

}