<?php

namespace WPSpeedo_Team;

if ( ! defined( 'ABSPATH' ) ) exit;

abstract class Style_Manager extends Attribute_Manager {

    public $settings = [];

    public $style = [];

    public $style_tablet = [];

    public $style_small_tablet = [];

    public $style_mobile = [];

    public function add_style_row( $selector, $out_css_props, $device = '' ) {

        $style = $device ? "style_$device" : 'style';

        if ( array_key_exists($selector, $this->$style) && empty($this->$style[$selector]) ) {
            $this->$style[$selector] = [];
        }

        $this->$style[$selector][] = $out_css_props;

    }

    public function add_style( $selector, $css_property, $setting_key, $field = 'value', $device = '' ) {

        $setting_key = $device ? $setting_key .'_'. $device : $setting_key;

        if ( ! is_array($field) ) $field = [$field];

        $setting = $this->get_setting( $setting_key, $field );

        $out_css_props = '';

        try {
            $out_css_props = preg_replace_callback( '/{{([^}| ]*)}}/', function( $matches ) use ($setting) {

                $parsed_value = '';
    
                if ( $matches[1] && isset( $setting[ $matches[1] ] ) ) {
                    $parsed_value = $setting[ $matches[1] ];
                }

                return $parsed_value;
    
            }, $css_property );

        } catch ( \Exception $e ) { return; }

        $this->add_style_row( $selector, $out_css_props, $device );

    }

    public function add_responsive_style( $selector, $css_property, $setting_key, $field = 'value' ) {

        $this->add_style( $selector, $css_property, $setting_key, $field );
        $this->add_style( $selector, $css_property, $setting_key, $field, 'tablet' );
        $this->add_style( $selector, $css_property, $setting_key, $field, 'small_tablet' );
        $this->add_style( $selector, $css_property, $setting_key, $field, 'mobile' );

    }

    public function add_background_style( $selector, $setting_base, $prop, $device = '' ) {

        $has_bg =  $this->get_setting( $setting_base . 'background' );
        $type =    $this->get_setting( $setting_base . 'type' );
        $clr =     $this->get_setting( $setting_base . 'color' );

        if ( $has_bg === 'yes' && !empty($type) && !empty($clr) ) {

            if ( $type === 'classic' ) {

                $this->add_style_row( $selector, sprintf( '%s:%s', $prop, $clr ), $device );

            } else {

                $grad_type = $this->get_setting( $setting_base . 'gradient_type' );

                if ( !empty($grad_type) ) {
                    
                    $clr_stop =     $this->get_setting( $setting_base . 'color_stop' );
                    $clr_b =        $this->get_setting( $setting_base . 'color_b' );
                    $clr_b_stop =   $this->get_setting( $setting_base . 'color_b_stop' );
                    
                    $color_start = sprintf( '%s %s', $clr, $clr_stop . '%' );
                    $color_end = sprintf( '%s %s', $clr_b, $clr_b_stop . '%' );
                    
                    if ( $grad_type === 'linear' ) {
                        $grad_angle = $this->get_setting( $setting_base . 'gradient_angle' );
                        $gradient = sprintf( 'linear-gradient(%s, %s, %s)', $grad_angle . 'deg', $color_start, $color_end );
                    } else {
                        $grad_pos = $this->get_setting( $setting_base . 'gradient_position' );
                        $gradient = sprintf( 'radial-gradient(at %s, %s, %s)', $grad_pos, $color_start, $color_end );
                    }
    
                    $this->add_style_row( $selector, sprintf( '%s:%s', $prop, $gradient ), $device );

                }

            }

        }

    }

    public function validate_style_props( Array $style_props ) {
        $_style_props = [];
        foreach ( $style_props as $style_prop ) {
            $value = explode( ':', $style_prop, 2 )[1];
            if ( empty( trim($value) ) ) continue;
            $_style_props[] = $style_prop;
        }
        return $_style_props;
    }

    public function build_css( $device = '' ) {

        $css = '';

        $style = $device ? "style_$device" : 'style';

        $style = $this->$style;

        if ( empty($style) ) return '';

        foreach ( $style as $selector => $style_props ) {
            $style_props = $this->validate_style_props( $style_props );
            $css.= sprintf( '%s{%s}', $selector, implode(';', $style_props) );
        }

        $css = str_replace( ';;', ';', $css );
        $css = str_replace( ';}', '}', $css );

        if ( $device == 'tablet' ) $css = sprintf( '@media screen and (max-width: 1024px){%s}', $css );
        if ( $device == 'small_tablet' ) $css = sprintf( '@media screen and (max-width: 767px){%s}', $css );
        if ( $device == 'mobile' ) $css = sprintf( '@media screen and (max-width: 480px){%s}', $css );

        return $css;

    }

    public function get_setting( String $setting, $field = 'value' ) {

        if ( ! isset( $this->settings[$setting] ) ) return null;

        $setting = $this->settings[$setting];

        if ( is_array($field) ) {
            return array_intersect_key( $setting, array_flip($field) );
        }

        if ( $field == 'all' ) return $setting;
        
        return $setting[$field];

    }

    public function _generate_css() {
        $this->generate_css();
        return $this->build_css() . $this->build_css( 'tablet' ) .  $this->build_css( 'small_tablet' ) . $this->build_css( 'mobile' );
    }

    public function print_css( $style_attribute_string ) {
        $css = $this->_generate_css();
        if ( !empty($css) ) printf( '<style %s>%s</style>', $this->get_attribute_string( $style_attribute_string ), $css );
    }

    abstract public function generate_css();

}