<?php

namespace WPSpeedo_Team;

if ( !defined( 'ABSPATH' ) ) {
    exit;
}
class Plugin_Hooks
{
    public function __construct()
    {
        add_filter( 'template_include', [ $this, 'maybe_load_dynamic_template' ] );
        add_action( 'init', [ $this, 'maybe_flush_rewrite_rules' ], PHP_INT_MAX );
        add_action( 'wpspeedo_team/before_single_team', [ $this, 'before_single_team' ] );
        add_action( 'wpspeedo_team/before_wrapper_inner', [ $this, 'before_wrapper_inner' ] );
        add_action( 'wpspeedo_team/after_wrapper_inner', [ $this, 'after_wrapper_inner' ] );
    }
    
    function single_page_few_info()
    {
        return [
            '_mobile',
            '_telephone',
            '_email',
            '_website'
        ];
    }
    
    function maybe_load_dynamic_template( $template )
    {
        
        if ( is_singular( Helpers::post_type_name() ) ) {
            add_filter( 'wpspeedo_team/few_info', [ $this, 'single_page_few_info' ] );
            return Helpers::load_template( 'template-single.php' );
        }
        
        if ( is_post_type_archive( Helpers::post_type_name() ) ) {
            return Helpers::load_template( 'template-archive.php' );
        }
        if ( is_tax( Helpers::archive_enabled_taxonomies() ) ) {
            return Helpers::load_template( 'template-archive.php' );
        }
        return $template;
    }
    
    function maybe_flush_rewrite_rules()
    {
        if ( get_option( Helpers::rewrite_flush_key(), false ) === false ) {
            flush_rewrite_rules();
        }
    }
    
    public function before_wrapper_inner( $widget_loader )
    {
        $display_type = $widget_loader->get_setting( 'display_type' );
        $card_action = $widget_loader->get_setting( 'card_action' );
        if ( $display_type == 'filter' && $card_action != 'expand' ) {
            include Helpers::load_template( 'partials/template-filter-header.php' );
        }
    }
    
    public function before_single_team( $widget_loader )
    {
        $terms_classes = Helpers::get_post_term_slugs( Helpers::group_taxonomy_name() );
        if ( !empty($terms_classes) ) {
            $widget_loader->add_attribute( 'single_item_col_' . get_the_ID(), 'class', $terms_classes );
        }
    }
    
    public function after_wrapper_inner( $widget_loader )
    {
        $card_action = $widget_loader->get_setting( 'card_action' );
        $display_type = $widget_loader->get_setting( 'display_type' );
        
        if ( $display_type === 'carousel' && $card_action !== 'expand' ) {
            if ( $widget_loader->get_setting( 'navs' ) ) {
                ?>
                <div class="wps-team--carousel-navs">
                    <button class="swiper-button-prev"><i aria-hidden="true" class="fas fa-chevron-left"></i></button>
                    <button class="swiper-button-next"><i aria-hidden="true" class="fas fa-chevron-right"></i></button>
                </div>
            <?php 
            }
            if ( $widget_loader->get_setting( 'dots' ) ) {
                ?>
                <div class="swiper-pagination"></div>
            <?php 
            }
        }
    
    }

}