<?php

namespace WPSpeedo_Team;

if ( !defined( 'ABSPATH' ) ) {
    exit;
}
class Shortcode_Loader extends Style_Manager
{
    public  $mode ;
    public  $id ;
    public  $posts = array() ;
    function __construct( $args )
    {
        $this->id = $args['id'];
        $this->mode = $args['mode'];
        $this->settings = $args['settings'];
        $this->set_attributes();
        
        if ( $this->mode == 'preview' ) {
            $this->load_editor_template();
        } else {
            $this->load_public_template();
        }
        
        $this->add_attribute( 'wpspeedo-team--css', 'class', 'wpspeedo-team--customized' );
        $this->print_css( 'wpspeedo-team--css' );
    }
    
    public function get_shortcode_id()
    {
        return 'wps-widget--team-' . $this->id;
    }
    
    public function set_attributes()
    {
        $theme = $this->get_setting( 'theme' );
        $display_type = $this->get_setting( 'display_type' );
        $card_action = $this->get_setting( 'card_action' );
        if ( $card_action == 'expand' && in_array( $display_type, [ 'carousel', 'filter' ] ) ) {
            $display_type = 'grid';
        }
        $this->add_attribute( 'wrapper', 'id', $this->get_shortcode_id() );
        $this->add_attribute( 'wrapper', 'class', [
            'wps-container wps-widget wps-widget--team',
            'wps-team-theme--' . $theme,
            'wps-team-type--' . $display_type,
            'wps-team-card-action--' . $card_action
        ] );
        $this->add_attribute( 'wrapper_inner', 'class', [ 'wps-container--inner' ] );
        $this->add_attribute( 'single_item_row', 'class', 'wps-row' );
        $this->add_attribute( 'single_item_col', 'class', 'wps-col' );
        
        if ( $display_type == 'carousel' ) {
            $this->add_attribute( 'wrapper_inner', 'class', 'swiper' );
            $this->add_attribute( 'single_item_row', 'class', 'swiper-wrapper' );
            $this->add_attribute( 'single_item_col', 'class', 'swiper-slide' );
            if ( wp_validate_boolean( $this->get_setting( 'dots' ) ) ) {
                $this->add_attribute( 'wrapper', 'class', 'wps-team--carousel-has-dots' );
            }
            if ( wp_validate_boolean( $this->get_setting( 'navs' ) ) ) {
                $this->add_attribute( 'wrapper', 'class', 'wps-team--carousel-has-navs' );
            }
        }
        
        
        if ( in_array( $display_type, [ 'carousel' ] ) ) {
            $carousel_settings = [
                'columns'                   => (int) $this->get_setting( 'columns' ),
                'columns_tablet'            => (int) $this->get_setting( 'columns_tablet' ),
                'columns_small_tablet'      => (int) $this->get_setting( 'columns_small_tablet' ),
                'columns_mobile'            => (int) $this->get_setting( 'columns_mobile' ),
                'gap'                       => (int) $this->get_setting( 'gap' ),
                'gap_tablet'                => (int) $this->get_setting( 'gap_tablet' ),
                'gap_small_tablet'          => (int) $this->get_setting( 'gap_small_tablet' ),
                'gap_mobile'                => (int) $this->get_setting( 'gap_mobile' ),
                'gap_vertical'              => (int) $this->get_setting( 'gap_vertical' ),
                'gap_vertical_tablet'       => (int) $this->get_setting( 'gap_vertical_tablet' ),
                'gap_vertical_small_tablet' => (int) $this->get_setting( 'gap_vertical_small_tablet' ),
                'gap_vertical_mobile'       => (int) $this->get_setting( 'gap_vertical_mobile' ),
                'speed'                     => (int) $this->get_setting( 'speed' ),
                'dots'                      => wp_validate_boolean( $this->get_setting( 'dots' ) ),
                'navs'                      => wp_validate_boolean( $this->get_setting( 'navs' ) ),
                'loop'                      => wp_validate_boolean( $this->get_setting( 'loop' ) ),
            ];
            $this->add_attribute( 'wrapper', 'data-carousel-settings', json_encode( $carousel_settings ) );
        }
    
    }
    
    public function load_editor_template()
    {
        ?>
        <!DOCTYPE html>
        <html <?php 
        language_attributes();
        ?>>
            <head>
                <meta charset="<?php 
        bloginfo( 'charset' );
        ?>">
                <meta name="viewport" content="width=device-width, initial-scale=1">
                <?php 
        wp_head();
        ?>
            </head>
            <body <?php 
        body_class();
        ?>>
                <div class='wps-widget--preview_wrapper'>
                    <?php 
        $this->load_template();
        ?>
                </div>
                <?php 
        wp_footer();
        ?>
            </body>
        </html>
        <?php 
    }
    
    public function load_public_template()
    {
        $this->load_template();
    }
    
    public function set_posts()
    {
        $query_args = [
            'orderby' => $this->get_setting( 'orderby' ),
            'order'   => $this->get_setting( 'order' ),
        ];
        if ( !$this->get_setting( 'show_all' ) ) {
            $query_args['posts_per_page'] = (int) $this->get_setting( 'limit' );
        }
        $this->posts = Helpers::get_posts( $query_args );
    }
    
    public function get_posts()
    {
        return $this->posts;
    }
    
    public function load_template()
    {
        $theme = $this->get_setting( 'theme' );
        $card_action = $this->get_setting( 'card_action' );
        $display_type = $this->get_setting( 'display_type' );
        if ( $card_action == 'expand' && in_array( $display_type, [ 'carousel', 'filter' ] ) ) {
            $display_type = 'grid';
        }
        $this->set_posts();
        if ( !in_array( $theme, Helpers::get_active_themes() ) ) {
            $theme = 'style-one';
        }
        $this->load_template_hooks();
        include Helpers::load_template( "template-{$theme}.php" );
        wp_reset_query();
    }
    
    public function load_template_hooks()
    {
        if ( $this->mode !== 'preview' && $this->id && (current_user_can( 'editor' ) || current_user_can( 'administrator' )) ) {
            add_action( 'wpspeedo_team/before_wrapper_inner', [ $this, 'add_shortcode_edit_link' ] );
        }
    }
    
    public function add_shortcode_edit_link()
    {
        ?>
            <div class="wps-widget--edit-link">
                <a target="_blank" href="<?php 
        echo  admin_url( "/admin.php?page=wps-team#/shortcode/{$this->id}" ) ;
        ?>">
                    <i class="fas fa-pencil-alt"></i>
                    <span class="wps-widget--edit-link-popup">Only <strong>Admin</strong> & <strong>Editor</strong> can see this link</span>
                </a>
            </div>
        <?php 
    }
    
    public function shortcode_selector()
    {
        return '#' . $this->get_shortcode_id() . '.wps-widget--team';
    }
    
    public function generate_css()
    {
        $this->add_responsive_style(
            $this->shortcode_selector(),
            '--wps-container-width: {{value}}{{unit}}',
            'container_width',
            [ 'value', 'unit' ]
        );
        $this->add_responsive_style( $this->shortcode_selector(), '--wps-item-col-gap-alt: calc(-{{value}}px/2)', 'gap' );
        $this->add_responsive_style( $this->shortcode_selector(), '--wps-item-col-gap: calc({{value}}px/2)', 'gap' );
        $this->add_responsive_style( $this->shortcode_selector(), '--wps-item-col-width: calc(100%/{{value}})', 'columns' );
        $this->add_background_style( $this->shortcode_selector(), 'item_background_', '--wps-item-bg-color' );
        $this->add_responsive_style( $this->shortcode_selector(), '--wps-title-color: {{value}}', 'title_color' );
        $this->add_responsive_style( $this->shortcode_selector(), '--wps-title-color-hover: {{value}}', 'title_color_hover' );
        $this->add_responsive_style( $this->shortcode_selector(), '--wps-desig-color: {{value}}', 'designation_color' );
        $this->add_responsive_style( $this->shortcode_selector(), '--wps-text-color: {{value}}', 'desc_color' );
        $this->add_responsive_style( $this->shortcode_selector() . '.wps-team--divider', '--wps-divider-bg-color: {{value}}', 'divider_color' );
        do_action( 'wpspeedo_team/generate_css', $this );
    }

}